package Format;

use strict;
use warnings;

=head1 format.pm (i18n)

Povides functions for localized data formating.

Suports formats:

    number
	formated using LC_NUMERIC with grouping
	(like SYSv2 printf "%'d"); 

    bytes
	formated like number;

    date datetime
	formated with strftime;
	
    time
	relative time in seconds, formated like strftime;

    duration
	number of mseconds, formatted into H:M:S.msec;

    host
	puts clean IP in brackets [ ], sorted by domains or IP;
	Has nothing to do with i18n, but just a good place to put.
	
    undef
	returned as is.

    else
	anything else is put into quotes. I dont know what for.


TODO: Maybe to format 'bytes' as --human-readable ?
      Actually, mixed G/M/K in same table look confusing.

=cut

use vars qw(%lc_numeric %typeformats @EXPORT_OK);

use POSIX;
use Exporter;

@Format::ISA = ('Exporter');

@EXPORT_OK = qw(
	comp
	i18n_format
	i18n_sort
	strfduration
	strfnum
);

%lc_numeric = (
    grouping => 3,
    thousands_sep => "'",
    decimal_point => "."
    );

%typeformats = (
    'date'	=> q(%x),
    'time'	=> q(%X),
    'datetime'	=> q(%c)
);

$Format::NAME = 'Format';
$Format::VERSION = '0.1.0';

print "\nFormat.pm (Format) loaded. Returning." if ($::DEBUG & 0x01);

#### BEGIN

# seems like broken locale support in 5.005
unless( $] <= 5.00503 ) {
	my $lconv = POSIX::localeconv();
	if( defined $lconv->{'grouping'} ) {
		my @grp=unpack('C*',$lconv->{'grouping'});
		$lc_numeric{'grouping'} = $grp[0] 
		}
	$lc_numeric{'thousands_sep'} = $lconv->{'thousands_sep'} if( defined $lconv->{'thousands_sep'});
	$lc_numeric{'decimal_point'} = $lconv->{'decimal_point'} if( defined $lconv->{'decimal_point'});
}

#### END of BEGIN

=head1 FUNCTIONS

=cut

sub strfnum
{
    my ($i,$f) = split(/\./,shift);
    my $str="";
    if( defined $lc_numeric{'grouping'} ) {
	my $thousands_sep=$lc_numeric{'thousands_sep'};
	while($i =~ s/(\d)(\d{$lc_numeric{grouping}}($|$thousands_sep))/$1$thousands_sep$2/) {};
	}
    $str=$i;

    if( defined $f ) {
    	$str .= $lc_numeric{'decimal_point'}.$f;
	}

    return $str;
}

sub strfduration
{
    my $elapsed = shift;
    my ($hours,$mins,$secs,$msecs);
    $msecs=$elapsed;
    return sprintf(".%03d",$msecs) if( $msecs < 1000 );
    $secs = int($msecs / 1000);
    $msecs = $msecs % 1000;
    return sprintf("%02d.%03d",$secs,$msecs) if( $secs < 60 );
    $mins = int($secs / 60);
    $secs = $secs % 60;
    return sprintf("%02d:%02d.%03d",$mins,$secs,$msecs) if( $mins < 60 );
    $hours = int($mins / 60);
    $mins = $mins % 60;
    return sprintf("%d:%02d:%02d.%03d",$hours,$mins,$secs,$msecs);
}

=head2 i18n_format ($TYPE,$VALUE)

Formats VALUE according to given type.

=cut

sub i18n_format
{
    my ($type,$val) = @_;
    return $val unless($type);
    return undef unless(defined $val);
    if( $type eq 'number' ) { return strfnum($val); }
    elsif( $type eq 'bytes' ) { return strfnum($val); }
    elsif( $type eq 'date' ) { return POSIX::strftime($typeformats{'date'},(localtime $val)); }
    elsif( $type eq 'time' ) { return POSIX::strftime($typeformats{'time'},(gmtime $val)); }
    elsif( $type eq 'datetime' ) { return POSIX::strftime($typeformats{'datetime'},(localtime $val)); }
    elsif( $type eq 'duration' ) { return strfduration($val); }
    elsif( $type eq 'host' ) { $val =~ s/^[0-9\.]+$/[$&]/; return $val; }
    return "\"$val\"";
}


=head2 i18n_sort ($type, @ARRAY)

For type 'host' sorts by domains or IP;
For anything else - numerically;
For empty type - stringly;
For undef - by default sort;

=cut

sub i18n_sort
{
    my($type,@array)=@_;
    return sort @array			unless($type);
    return sort hostsort @array		if( $type eq 'host' );
    return sort { $a cmp $b } @array	if( $type eq '' );
    return sort { $a <=> $b } @array;   # all other registered types
}

=head2 comp ($type, $a, $b)

Compares values according to type, that is,
either numerically, stringly or domainly.

=cut

sub comp
{
	my ($type, $l, $r) = @_;
	
	return ($l cmp $r) unless ($type);

	if ($type eq 'host')
	{
		return (hostcmp ($l, $r));
	}
	
	return ($l <=> $r); # all other registered types
}

sub hostcmp
{
	my ($l, $r) = @_;

	# ip adresses
	if ($l !~ m/[^0-9\.]/ and $r !~ m/[^0-9\.]/)
	{
		my $ul = sprintf ("%03u%03u%03u%03u", split (m/\./, $l));
		my $ur = sprintf ("%03u%03u%03u%03u", split (m/\./, $r));

		return ($ul <=> $ur);
	}
	else
	{
		my $ul = join ('.', reverse (split (m/\./, $l)));
		my $ur = join ('.', reverse (split (m/\./, $r)));

		return ($l cmp $r);
	}
}

=head1 AUTHOR

QMax E<lt>qmax@mail.ruE<gt>

=cut

1;
